﻿##### Scriptinfo ##################################################################################
# Scriptreihe:      Show-EncryptionAsymmetric
# Datum:            2017-09-19
# Version:          V1.00
# Programmierer:    Stephan Walther
###################################################################################################
    
# Funktionen    
    Function Encrypt-String {
        <#
        .Synopsis
            Diese Funktion verschlüsselt einen Text asymmetrisch mit einem PublicKey aus einem Zertifikat.
        .DESCRIPTION
            Der EingabeText wird mit einem PublicKey asymmetrisch verschlüsselt. Nur mit dem zum PublicKey
            gehörigen PrivateKey ist die Entschlüsselung möglich. Es wird also ein passendes Zertifikat 
            vorausgesetzt.

            Die Verschlüsselung kann mit Decrypt-String umgekehrt werden.
        .Parameter String
            Das ist der Eingabetext - bestehend aus einem oder mehrerer Elemente. Diese können direkt oder
            über die Pipeline eingegeben werden.

            Der Parameter String schließt SecureInput aus.
            Der Parameter String benötigt CertThumbprint oder PublicKeyPath als Angabe zum Zertifikat,
            dass verwendet werden soll. Alternativ kann keiner der beiden Parameter verwendet werden. Dann
            sucht die Funktion automatisch nach einer cer-Datei im aktuellen Verzeichnis. Sind mehrere 
            Dateien vorhanden, wird die erste verwendet.
        .Parameter SecureInput
            Mit SecureInput wird von der Funktion ein Eingabefenster erzeugt, in dem die Stringeingabe
            verdeckt vorgenommen werden kann.

            Der Parameter SecureInput schließt String als Eingabe aus.
            Der Parameter SecureInput benötigt CertThumbprint oder PublicKeyPath als Angabe zum Zertifikat,
            dass verwendet werden soll. Alternativ kann keiner der beiden Parameter verwendet werden. Dann
            sucht die Funktion automatisch nach einer cer-Datei im aktuellen Verzeichnis. Sind mehrere 
            Dateien vorhanden, wird die erste verwendet.
        .Parameter EncryptedFilePath
            Optional kann die Ausgabe des verschlüsselten Strings in eine Textdatei umgeleitet werden. Der
            Pfad wird mit dem Parameter EncryptedFilePath angegeben. Das besondere an der Datei: die erste
            Zeile enthält den CertThumbprint des verwendeten Zertifikates. Dies ermöglicht eine einfachere
            Entschlüsselung mit Decrypt-String.            
        .Parameter CertThumbprint
            Für die Verschlüsselung ist ein PublicKey eines Zertifikates erforderlich. Über CertThumbPrint
            kann der ThumbPrint des Zertifikates angegeben werden. Die Funktion sucht dann im Benutzer-
            und Computer-Zertifikatspeicher nach dem Zertifikat.

            CertThumbprint schließt PublicKeyPath aus.
        .Parameter PublicKeyPath
            Für die Verschlüsselung ist ein PublicKey eines Zertifikates erforderlich. Über PublicKeyPath
            kann der Pfad zu einer *.cer-Datei eines Zertifikates angegeben werden.

            PublicKeyPath schließt CertThumbprint aus.       
        .EXAMPLE
            # VORBEREITUNG FÜR ALLE BEISPIELE!!!

            # SpieleDatei erzeugen
                Get-ChildItem -Path c:\ | Out-File -FilePath c:\Klartext.txt

            # ermittle vorhandene Cert-Thumbprints
                Get-ChildItem -Path Cert:\CurrentUser\My
                Get-ChildItem -Path Cert:\Localmachine\My

            # CER exportieren
                Get-ChildItem -Path Cert:\Localmachine\My |
                    Where-Object {$_.ThumbPrint -eq 'B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F'} |
                        Export-Certificate -Type CERT -FilePath c:\my.cer
        .EXAMPLE 
            # einzelnen String verschlüsseln
            
            Encrypt-String -String 'Pa$$w0rd' -Verbose
            Encrypt-String -String 'Pa$$w0rd' -verbose -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F 
            Encrypt-String -String 'Pa$$w0rd' -verbose -PublicKeyPath C:\my.cer
        .Example
            # Inhalt einer Textdatei verschlüsselt anzeigen
            
            Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F            
        .Example
            # Inhalt einer Textdatei in eine Datei verschlüsseln (ThumbPrint ist die erste Zeile im File!)
            
            Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt                                   
        .EXAMPLE
            # Info in Variable verschlüssel und entschlüsseln    
            
            # verschlüsseln
                $verschluesselt = Get-Content -Path c:\Klartext.txt |
                    Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

            # entschlüsseln (variante 1)
                $verschluesselt | Decrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

            # entschlüsseln (Variante 2)
                Decrypt-String -String $verschluesselt -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F
        .EXAMPLE
             # Inhalt einer verschlüsselten Datei lesen 
             
             # verschlüsselte Datei erzeugen
                Get-Content -Path c:\Klartext.txt | 
                    Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt
            
             # verschlüsselte Datei lesen (Variante 1)
                Decrypt-String -EncryptedFilePath C:\verschluesselt.txt -Verbose

             # verschlüsselte Datei lesen (Variante 2)
                Get-Item -Path C:\verschluesselt.txt | Decrypt-String
        .EXAMPLE
             # Inhalt mehrerer verschlüsselter Dateien lesen 
             
             # verschlüsselte Datei erzeugen (Beispiel: 2 unterschiedliche Zertifikate verwendet!)
                Encrypt-String -String 'geheim-A' -CertThumbprint 842B2E07C4F4D382F3AA252E7080427C2688C74D -EncryptedFilePath c:\verschluesselt-A.txt                
                Encrypt-String -String 'geheim-B' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-B.txt
                Encrypt-String -String 'geheim-C' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-C.txt
            
             # verschlüsselte Datei lesen (Variante 1)
                Get-ChildItem -Path "C:\verschluesselt-*.txt" | Decrypt-String -Verbose                          
        .EXAMPLE
            # Verschlüsselung einer CSV
            
            # Daten erzeugen
                $CSV_DATA = Get-ChildItem -Path c:\ | 
                    Select-Object -Property Name,length,Lastwritetime,mode | 
                        ConvertTo-Csv -Delimiter ';' -NoTypeInformation

            # Daten in Datei verschlüsseln
                Encrypt-String -String $CSV_DATA -PublicKeyPath C:\my.cer -EncryptedFilePath c:\csv_data.txt
        
            # Daten aus Datei entschlüsseln und formatieren
                Get-Item -Path C:\csv_data.txt | 
                    Decrypt-String |
                        ConvertFrom-Csv -Delimiter ';'                          
        #.EXAMPLE
            # Kennwortverschlüsselung für eine Anmeldung
            
            # Verschlüsselung des Kennwortes
                Encrypt-String -SecureInput -EncryptedFilePath c:\SecurePWD.txt

            # Entschlüsselung des Kennwortes und Aufbau einer Anmeldung 

             $pass = Decrypt-String -EncryptedFilePath C:\SecurePWD.txt | ConvertTo-SecureString -AsPlainText -Force
             $user = 'domain\user'
             $cred = New-Object System.Management.Automation.Pscredential -Argumentlist $user,$pass 

             Enter-PSSession -ComputerName servername -Credential $cred                             
        .NOTES
           Programmierer: Stephan Walther        
        #>

        [cmdletbinding()]
        param (
            [parameter(Mandatory=$false,ParameterSetName='Set1',ValueFromPipeline=$true)] [string[]] $String,
            [parameter(Mandatory=$false,ParameterSetName='Set2')]                         [switch]   $SecureInput,            
            [parameter(Mandatory=$false)]                                                 [string]   $CertThumbprint,
            [parameter(Mandatory=$false)]                                                 [string]   $PublicKeyPath,
            [parameter(Mandatory=$false)]                                                 [string]   $EncryptedFilePath = ''
        )

        begin {
            # Variablen
                $FehlerFrei = $true
                [string[]] $StringEncrypted = @()

                Get-Variable | Where-Object {$_.name -eq 'string2'} | Remove-Variable -Scope global
                New-Variable -Name String2 -Value @() -Scope global
                
            # Vorprüfung
                if ($SecureInput -eq $false -and $String -eq @()) {
                    Write-Host "Bitte geben Sie einen String an!" -ForegroundColor Yellow
                    $FehlerFrei = $false       # keine weitere Aktion
                } 
                $usedCertParameters = 0
                if ($CertThumbprint.Length -ne '') {$usedCertParameters += 1}
                if ($PublicKeyPath.Length -ne '')  {$usedCertParameters += 1}                

                if ($usedCertParameters -gt 1) {
                    Write-Host "Bitte nur CertThumbprint ODER PublicKeyPath ODER keinen von beiden verwenden!" -ForegroundColor Yellow
                    $FehlerFrei = $false       # keine weitere Aktion
                } else {
                    # keine CertiInfo -> suche *.cer im aktuellen Verzeichnis
                        if ($CertThumbprint -eq '' -and $PublicKeyPath -eq '') {                    
                            $PublicKeyPath2 = Get-ChildItem -Filter *.cer | Select-Object -First 1 -ExpandProperty fullname                    
                                      
                            if ($PublicKeyPath2 -eq $null) {
                                Write-Host "Es wurde kein CertFile in '$((Get-Location).Path)' gefunden. Bitte verwende den Parameter PublicKeyPath oder CertThumbPrint!" -ForegroundColor Yellow
                                $FehlerFrei = $false       # keine weitere Aktion
                            } else {
                                $Certificate = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2($PublicKeyPath2)
                                Write-Verbose "verwende gefundenes CertFile '$PublicKeyPath2' mit ThumbPrint $($Certificate.Thumbprint)"
                            }
                        } else {
                            # Certificate via Thumbprint oder PublicKeyPath?
                                if ($CertThumbprint -eq '') {
                                    if (Test-Path -Path $PublicKeyPath) {
                                        $Certificate = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2($PublicKeyPath)
                                        Write-Verbose "verwende angegebenes CertFile '$PublicKeyPath' mit ThumbPrint $($Certificate.Thumbprint)"
                                    } else {
                                        Write-Host "Die Datei '$PublicKeyPath' wurde nicht gefunden!" -ForegroundColor Yellow
                                        $FehlerFrei = $false       # keine weitere Aktion
                                    }                 
                                } else {
                                    # suche Zertifikat in allen möglichen Stores (Berechtigungsfehler werden ignoriert
                                        $Certificate = @()
                                        $Certificate += Get-ChildItem -Path "Cert:\CurrentUser\My"  -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint}
                                        $Certificate += Get-ChildItem -Path "Cert:\LocalMachine\My" -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint}

                                    # Zertifikat gefunden?
                                        if (-not $Certificate) {
                                            Write-Host "Das Zertifikat mit dem ThumbPrint $CertThumbprint wurde nicht gefunden!" -ForegroundColor Yellow
                                            $FehlerFrei = $false   # keine weitere Aktion
                                        } else {
                                            Write-Verbose "verwende Certificate aus LocalStore mit ThumbPrint $($Certificate.Thumbprint)"
                                        }
                                }   
                        }     
                }    
        }
        
        process {
            if ($FehlerFrei) {

                # SecureString?
                    if ($SecureInput) {
                        [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") 
                        [void] [System.Reflection.Assembly]::LoadWithPartialName("System.Drawing") 
                        [System.Reflection.Assembly]::LoadWithPartialName('Microsoft.VisualBasic') | Out-Null
                        [System.Windows.Forms.Application]::EnableVisualStyles()

                        $form_Main = New-Object System.Windows.Forms.Form 
                        $form_Main.Text = "Encrypt-String"
                        $form_Main.Size = '440,100'
                        $form_Main.StartPosition = "CenterScreen"
                        $form_Main.Topmost = $false
                        $form_Main.KeyPreview = $True
       
                        $lbl_SecureString = New-Object System.Windows.Forms.Label
                        $lbl_SecureString.Location = '10,10'
                        $lbl_SecureString.Size = '50,20'
                        $lbl_SecureString.Text = "String:"
                        $form_Main.Controls.Add($lbl_SecureString) 

                        $obj_SecureString = New-Object System.Windows.Forms.MaskedTextbox
                        $obj_SecureString.Location = '60,10'
                        $obj_SecureString.Size = '290,20'
                        $obj_SecureString.text = "" 
                        $obj_SecureString.passwordChar = '*'
                        $form_Main.Controls.Add($obj_SecureString)
        
                        $Button_Setup = New-Object System.Windows.Forms.Button
                        $Button_Setup.Location = '360,10'
                        $Button_Setup.Size = '50,20'
                        $Button_Setup.Text = "OK"                                                                                                                        
                        $Button_Setup.Add_Click({    
                            $global:String2 = 
                            $global:String2 += $obj_SecureString.Text
                            $form_Main.Close()    
                        })    
                        $form_Main.Controls.Add($Button_Setup)
       
                        $form_Main.ResumeLayout()	
                        $InitialFormWindowState = $form_Main.WindowState
                        $form_Main.Add_Shown({
                            $form_Main.Activate()        
                            $form_Main.cursor = [System.Windows.Forms.Cursors]::Default            
                        })    	
	                    $form_Main.add_Load({
        
                        })
                        [void] $form_Main.ShowDialog()                            
                    } else {
                        $global:string2 = $String
                    }                

                # verschlüssele jeden String einzeln                    
                    $String2 | ForEach-Object {                             
                        $StringBytes      = [System.Text.Encoding]::UTF8.GetBytes($_)                        
                        $StringEncrypted += ($Certificate.PublicKey.Key.Encrypt($StringBytes,$true) -join ',')                        
                    }
            }
        }

        end {
            if ($FehlerFrei) {                
                if ($EncryptedFilePath -eq '') { 
                    # Ausgabe an Caller           
                        return $StringEncrypted
                } else {
                    # Ausgabe in Datei?

                    try {
                        # schreibe Certificate-Thumbprint als erste Zeile in Datei
                            $Certificate.Thumbprint | Out-File -FilePath $EncryptedFilePath

                        # schreibe alle encrypted Strings in Datei
                            $StringEncrypted | ForEach-Object {
                                $_ | Out-File -FilePath $EncryptedFilePath -Append
                            }
                    } 
                    catch {
                        $Fehler = $Error[0].Exception.Message
                        Write-Host "Fehler: $Fehler" -ForegroundColor Yellow
                    }
                }
            }    
        }
    }

    Function Decrypt-String {
        <#
        .Synopsis
            Diese Funktion entschlüsselt einen Text asymmetrisch mit einem PrivateKey aus einem Zertifikat.
        .DESCRIPTION
            Der EingabeText wird mit einem PrivateKey asymmetrisch entschlüsselt. Dieser muss zuvor mit
            dem zum PrivateKey gehörigen PublicKey verschlüsselt worden sein. Es wird also ein passendes 
            Zertifikat vorausgesetzt. Der PrivateKey muss sich im Zertifikatspeicher des Benutzers oder
            des Computers befinden. Der Benutzer benötigt entsprechende Zugriffsrechte.

           Die Verschlüsselung kann mit Encrypt-String vorgenommen werden.
        .Parameter EncryptedFile
            Mit diesem Parameter können ein oder mehrere Dateiobjekte an die Funktion übergeben werden. 
            Die Dateien müssen mit encrypt-String erzeugt worden sein, da decrypt-String eine entsprechende 
            Formatierung erwartet. Dabei kann jede Datei mit einem anderen PublicKey verschlüsselt worden 
            sein. Für jede Datei muss ein PrivateKey verfügbar sein. Es ist eine Übergabe direkt oder mit
            einer Pipeline möglich.

            Dieser Parameter schließt die Verwendung von EncryptedFilePath und String+CertThumbprint aus.
        .Parameter EncryptedFilePath    
            Dieser Parameter nimmt einen Pfad zu einer Datei als String entgegen. Diese Datei muss mit 
            encrypt-String erzeugt worden sein, da decrypt-String eine entsprechende Formatierung erwartet.

            Dieser Parameter schließt die Verwendung von EncryptedFile und String+CertThumbprint aus.
        .Parameter String
            Diesem Parameter nimmt einen oder mehrere verschlüsselte Strings direkt oder über die Pipeline 
            entgegen. Diese müssen sich alle mit dem gleichen PrivateKey entschlüsseln lassen. Der 
            PrvateKey ist über seinen Thumbprint über den Parameter CertThumbprint anzugeben.

            Dieser Parameter schließt die Verwendung von EncryptedFile und EncryptedFilePath aus.
        .Parameter CertThumbprint
            Nur in Verbindung mit Parameter String. Mit diesem Parameter geben Sie den ThumbPrint des
            Zertifikates mit dem PrivateKey an.
        .EXAMPLE
            # VORBEREITUNG FÜR ALLE BEISPIELE!!!

            # SpieleDatei erzeugen
                Get-ChildItem -Path c:\ | Out-File -FilePath c:\Klartext.txt

            # ermittle vorhandene Cert-Thumbprints
                Get-ChildItem -Path Cert:\CurrentUser\My
                Get-ChildItem -Path Cert:\Localmachine\My

            # CER exportieren
                Get-ChildItem -Path Cert:\Localmachine\My |
                    Where-Object {$_.ThumbPrint -eq 'B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F'} |
                        Export-Certificate -Type CERT -FilePath c:\my.cer
        .EXAMPLE 
            # einzelnen String verschlüsseln
            
            Encrypt-String -String 'Pa$$w0rd' -Verbose
            Encrypt-String -String 'Pa$$w0rd' -verbose -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F 
            Encrypt-String -String 'Pa$$w0rd' -verbose -PublicKeyPath C:\my.cer
        .Example
            # Inhalt einer Textdatei verschlüsselt anzeigen
            
            Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F            
        .Example
            # Inhalt einer Textdatei in eine Datei verschlüsseln (ThumbPrint ist die erste Zeile im File!)
            
            Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt                                   
        .EXAMPLE
            # Info in Variable verschlüssel und entschlüsseln    
            
            # verschlüsseln
                $verschluesselt = Get-Content -Path c:\Klartext.txt |
                    Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

            # entschlüsseln (variante 1)
                $verschluesselt | Decrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

            # entschlüsseln (Variante 2)
                Decrypt-String -String $verschluesselt -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F
        .EXAMPLE
             # Inhalt einer verschlüsselten Datei lesen 
             
             # verschlüsselte Datei erzeugen
                Get-Content -Path c:\Klartext.txt | 
                    Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt
            
             # verschlüsselte Datei lesen (Variante 1)
                Decrypt-String -EncryptedFilePath C:\verschluesselt.txt -Verbose

             # verschlüsselte Datei lesen (Variante 2)
                Get-Item -Path C:\verschluesselt.txt | Decrypt-String
        .EXAMPLE
             # Inhalt mehrerer verschlüsselter Dateien lesen 
             
             # verschlüsselte Datei erzeugen (Beispiel: 2 unterschiedliche Zertifikate verwendet!)
                Encrypt-String -String 'geheim-A' -CertThumbprint 842B2E07C4F4D382F3AA252E7080427C2688C74D -EncryptedFilePath c:\verschluesselt-A.txt                
                Encrypt-String -String 'geheim-B' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-B.txt
                Encrypt-String -String 'geheim-C' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-C.txt
            
             # verschlüsselte Datei lesen (Variante 1)
                Get-ChildItem -Path "C:\verschluesselt-*.txt" | Decrypt-String -Verbose                          
        .EXAMPLE
            # Verschlüsselung einer CSV
            
            # Daten erzeugen
                $CSV_DATA = Get-ChildItem -Path c:\ | 
                    Select-Object -Property Name,length,Lastwritetime,mode | 
                        ConvertTo-Csv -Delimiter ';' -NoTypeInformation

            # Daten in Datei verschlüsseln
                Encrypt-String -String $CSV_DATA -PublicKeyPath C:\my.cer -EncryptedFilePath c:\csv_data.txt
        
            # Daten aus Datei entschlüsseln und formatieren
                Get-Item -Path C:\csv_data.txt | 
                    Decrypt-String |
                        ConvertFrom-Csv -Delimiter ';'                          
        #.EXAMPLE
            # Kennwortverschlüsselung für eine Anmeldung
            
            # Verschlüsselung des Kennwortes
                Encrypt-String -SecureInput -EncryptedFilePath c:\SecurePWD.txt

            # Entschlüsselung des Kennwortes und Aufbau einer Anmeldung 

             $pass = Decrypt-String -EncryptedFilePath C:\SecurePWD.txt | ConvertTo-SecureString -AsPlainText -Force
             $user = 'domain\user'
             $cred = New-Object System.Management.Automation.Pscredential -Argumentlist $user,$pass 

             Enter-PSSession -ComputerName servername -Credential $cred                             
        .NOTES
           Programmierer: Stephan Walther  
            
        #>
        [cmdletbinding()]
        param (
            [parameter(Mandatory=$true, ParameterSetName='Set1',ValueFromPipeline=$true)] [System.IO.FileInfo[]] $EncryptedFile,
            [parameter(Mandatory=$true, ParameterSetName='Set2')]                         [String]               $EncryptedFilePath,
            [parameter(Mandatory=$true, ParameterSetName='Set3',ValueFromPipeline=$true)] [String[]]             $String,
            [parameter(Mandatory=$true, ParameterSetName='Set3')]                         [string]               $CertThumbprint                     
        )

        begin {       
            [string[]] $StringClearText = @()
            [string[]] $EncryptedString = @()
            
            $FehlerFrei = $true                      
        } 

        process {
            if ($FehlerFrei) {
                try {     
                    $Dodecrypt = $true
                        
                    # Deklaration über String?
                        if ($String.count -gt 0) {
                            $EncryptedString  = @()
                            $EncryptedString  += $String                                                                    
                            Write-Verbose "Deklaration ueber String"

                            # suche Zertifikat in allen möglichen Stores (Berechtigungsfehler werden ignoriert
                                $Certificate = @()
                                $Certificate += Get-ChildItem -Path "Cert:\CurrentUser\My"  -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint}
                                $Certificate += Get-ChildItem -Path "Cert:\LocalMachine\My" -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint}

                            # Zertifikat gefunden?
                                if (-not $Certificate) {
                                    Write-Host "Das Zertifikat mit dem ThumbPrint $CertThumbprint wurde nicht gefunden!" -ForegroundColor Yellow
                                    $FehlerFrei = $false   # keine weitere Aktion
                                    $DoDecrypt = $false
                                } else {
                                    Write-Verbose "verwende Certificate aus LocalStore mit ThumbPrint $($Certificate.Thumbprint)"
                                }
                        }

                    # Deklaration über EncryptedFilePath?
                        if ($EncryptedFilePath.Length -gt 0) {
                            Write-Verbose "Deklaration ueber EncryptedFilePath"                                     
                            if (Test-Path -Path $EncryptedFilePath) {
                                $CertThumbprint2  =  Get-Content -Path $EncryptedFilePath | Select-Object -first 1 # 1. Zeile == ThumbPrint
                                $EncryptedString  += Get-Content -Path $EncryptedFilePath | Select-Object -Skip 1  # alle anderen == Strings
                                
                                # suche Zertifikat in allen möglichen Stores (Berechtigungsfehler werden ignoriert
                                    $Certificate = @()
                                    $Certificate += Get-ChildItem -Path "Cert:\CurrentUser\My"  -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint2}
                                    $Certificate += Get-ChildItem -Path "Cert:\LocalMachine\My" -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint2}
                                    
                                # Zertifikat gefunden?
                                    if (-not $Certificate) {
                                        Write-Host "Das Zertifikat mit dem ThumbPrint $CertThumbprint2 wurde nicht gefunden!" -ForegroundColor Yellow
                                        $FehlerFrei = $false   # keine weitere Aktion
                                    } else {
                                        Write-Verbose "verwende Certificate aus LocalStore mit ThumbPrint $($Certificate.Thumbprint)"
                                    }
                            } else {
                                Write-Host "Die Datei '$EncryptedFilePath' wurde nicht gefunden!" -ForegroundColor Yellow
                                $FehlerFrei = $false
                                $DoDecrypt = $false
                            }                               
                        } 
                    
                    # Deklaration über EncryptedFile?
                        if ($EncryptedFile.Count -gt 0) {
                            Write-Verbose "Deklaration ueber EncryptedFile = $EncryptedFile"

                            $EncryptedString = @()
                            $EncryptedFile | ForEach-Object {
                                # zerlege die Datei
                                    $CertThumbprint2  =  $_ | Get-Content | Select-Object -first 1 # 1. Zeile == ThumbPrint
                                    $EncryptedString  += $_ | Get-Content | Select-Object -Skip 1  # alle anderen == Strings 

                                # suche Zertifikat in allen möglichen Stores (Berechtigungsfehler werden ignoriert)
                                    $Certificate = @()
                                    $Certificate += Get-ChildItem -Path "Cert:\CurrentUser\My"  -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint2}
                                    $Certificate += Get-ChildItem -Path "Cert:\LocalMachine\My" -ErrorAction SilentlyContinue | Where-Object {$_.Thumbprint -eq $CertThumbprint2}

                                # Zertifikat gefunden?
                                    if (-not $Certificate) {
                                        Write-Host "Das Zertifikat mit dem ThumbPrint $CertThumbprint2 wurde nicht gefunden!" -ForegroundColor Yellow                                
                                        $DoDecrypt = $false
                                    } else {
                                        Write-Verbose "verwende Certificate aus LocalStore mit ThumbPrint $CertThumbprint2"
                                    }
                            }
                        }
                    
                    # Entschlüsselung starten?
                        if ($Dodecrypt) {
                            
                            $EncryptedString | ForEach-Object {  
                                $StringDecrypted  = $Certificate.PrivateKey.Decrypt(($_ -split ','),$true)                    
                                $StringClearText += [System.Text.Encoding]::UTF8.GetString($StringDecrypted) 
                            } 
                        }                           
                }
                catch {
                    $FehlerFrei = $false
                }
            }
        }

        end {
            if ($FehlerFrei) { 
                return $StringClearText    # Erfolg -> Ausgabe vom Klartext
            } else {
                $Fehler = $Error[0].Exception.Message
                Write-Host "Fehler: $Fehler" -ForegroundColor Yellow
            }
        }
    }
    
# Aufrufbeispiele
    # VORBEREITUNG FÜR ALLE BEISPIELE!!!
        Set-Location -Path c:
        cd\

        # SpieleDatei erzeugen
            Get-ChildItem -Path c:\ | Out-File -FilePath c:\Klartext.txt

        # ermittle vorhandene Cert-Thumbprints
            Get-ChildItem -Path Cert:\CurrentUser\My
            Get-ChildItem -Path Cert:\Localmachine\My

        # CER exportieren
            Get-ChildItem -Path Cert:\Localmachine\My |
                Where-Object {$_.ThumbPrint -eq 'B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F'} |
                    Export-Certificate -Type CERT -FilePath c:\my.cer

    # einzelnen String verschlüsseln            
        Encrypt-String -String 'Pa$$w0rd' -Verbose
        Encrypt-String -String 'Pa$$w0rd' -verbose -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F 
        Encrypt-String -String 'Pa$$w0rd' -verbose -PublicKeyPath C:\my.cer

    # Inhalt einer Textdatei verschlüsselt anzeigen            
        Get-Content -Path c:\Klartext.txt | 
            Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -Verbose      

    # Inhalt einer Textdatei in eine Datei verschlüsseln (ThumbPrint ist die erste Zeile im File!)            
        Get-Content -Path c:\Klartext.txt | 
            Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt                                   

    # Info in Variable verschlüssel und entschlüsseln                
        # verschlüsseln
            $verschluesselt = Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

        # entschlüsseln (variante 1)
            $verschluesselt | Decrypt-String -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

        # entschlüsseln (Variante 2)
            Decrypt-String -String $verschluesselt -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F

    # Inhalt einer verschlüsselten Datei lesen              
        # verschlüsselte Datei erzeugen
            Get-Content -Path c:\Klartext.txt | 
                Encrypt-String -PublicKeyPath C:\my.cer -EncryptedFilePath c:\verschluesselt.txt
            
        # verschlüsselte Datei lesen (Variante 1)
            Decrypt-String -EncryptedFilePath C:\verschluesselt.txt -Verbose

        # verschlüsselte Datei lesen (Variante 2)
            Get-Item -Path C:\verschluesselt.txt | Decrypt-String

    # Inhalt mehrerer verschlüsselter Dateien lesen              
        # verschlüsselte Datei erzeugen (Beispiel: 2 unterschiedliche Zertifikate verwendet!)
            Encrypt-String -String 'geheim-A' -CertThumbprint 842B2E07C4F4D382F3AA252E7080427C2688C74D -EncryptedFilePath c:\verschluesselt-A.txt                
            Encrypt-String -String 'geheim-B' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-B.txt
            Encrypt-String -String 'geheim-C' -CertThumbprint B771BFEA05EBE0EF53E287EC0BC64E89F8DBF30F -EncryptedFilePath c:\verschluesselt-C.txt
            
        # verschlüsselte Datei lesen (Variante 1)
            Get-ChildItem -Path "C:\verschluesselt-*.txt" | Decrypt-String -Verbose                          

    # Verschlüsselung einer CSV            
        # Daten erzeugen
            $CSV_DATA = Get-ChildItem -Path c:\ | 
                Select-Object -Property Name,length,Lastwritetime,mode | 
                    ConvertTo-Csv -Delimiter ';' -NoTypeInformation

        # Daten in Datei verschlüsseln
            Encrypt-String -String $CSV_DATA -PublicKeyPath C:\my.cer -EncryptedFilePath c:\csv_data.txt
        
        # Daten aus Datei entschlüsseln und formatieren
            Get-Item -Path C:\csv_data.txt | 
                Decrypt-String |
                    ConvertFrom-Csv -Delimiter ';'                          

    # Kennwortverschlüsselung für eine Anmeldung            
        # Verschlüsselung des Kennwortes
            Encrypt-String -SecureInput -EncryptedFilePath c:\SecurePWD.txt

        # Entschlüsselung des Kennwortes und Aufbau einer Anmeldung 
            $pass = Decrypt-String -EncryptedFilePath C:\SecurePWD.txt | ConvertTo-SecureString -AsPlainText -Force
            $user = 'domain\user'
            $cred = New-Object System.Management.Automation.Pscredential -Argumentlist $user,$pass 

            Enter-PSSession -ComputerName servername -Credential $cred                             

# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUhbRUuwqX4fD9cCE0UJed7mdc
# qZCgggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBQI/eviviAFYgtB1GhVjHQnXE+HeDANBgkqhkiG9w0BAQEF
# AASCAQBNGbRb6x9xN61kjdkYs8GC221pb45eO8f9ho1FRafi0nhOZKRNcQhqa5mo
# oY44XMWA2oigRkSMFHG5FYvYjGiGUWmKMNw/fWKZSS2SL/xlb3iwUnzZCBqK8wno
# KkwCi3ZcfSOAUnCQMREYlgXuKfaBwDiFrZu9saxiqaWabDCIt+ytbBl1wZApGpSr
# 2lRs3qAGdGYFk2XfSo71ubuevaqdxfMl0u5vYG6j0oqTQ1YZzjQ+ky9BBixtBRHt
# zNRNlJHXEjIPY7QroPddVds+uPtrrcC7oqdEIPtD1SWaLWaIUcEKS9l58TfcmZFv
# kQCb6+Akh52bFPw3Af5Npd5kam6D
# SIG # End signature block
